//+------------------------------------------------------------------+
//|                                          ay-BarsInOCRangeV02.mq4 |
//|                      Copyright  2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "@2011 ahmad.yani@hotmail.com"
#property link      "me@theroom"

#property indicator_chart_window

#define PREFIX "ay-RBar_"


//---- input parameters
extern string    TimeFrame      = "H1";
extern int       MinBarsInRange = 8
               , MaxBarsInRange = 20
               , Range          = 34
               , ExtendLines    = 16
               , BarsToScan     = 0
               , LineWidth      = 2;               
extern color     BoxColor       = Gray
               , LineColor      = Blue
               , TextColor      = Silver;
extern bool      ShowHHLLlines  = false;       
extern int       FontSize       = 6;


double fix_point;    
double fix_digits; 
int    giTf;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   
   fix_point  = Point;
   fix_digits = Digits;
   
   if (Point == 0.001 || Point == 0.00001 )
   { fix_point=Point*10; fix_digits= Digits -1; }
   
   giTf = getTFByName(TimeFrame);
   
   delObjs();   
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
//----
   delObjs();
//----
   return(0);
}
void delObjs()
{
   int objs = ObjectsTotal();
   string name;
   for(int cnt=ObjectsTotal()-1;cnt>=0;cnt--)
   {
      name=ObjectName(cnt);
      if (StringSubstr(name,0,StringLen(PREFIX)) == PREFIX) ObjectDelete(name); 
   }   
}  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   static bool startup = true;
   int limit, i, j;
   int counted_bars=IndicatorCounted();
   //---- check for possible errors
   if(counted_bars<0) return(-1);
   //---- the last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   //limit=Bars-counted_bars;      
   if (Period()>giTf) return(-1);
   
   limit = MathMin(Bars, BarsToScan);
   //limit = MathMax(limit, MinBarsInRange+1);
   if (limit == 0) limit = Bars;
   
   
   int last_ij;
   
   for (i=1; i<limit; i++)//*
   {      
      double h, l, ocrange, hh, ll, lasth, lastl, lastocr, lasthh, lastll;
      double aOCiR[5]; //{h,l,ocrange, hh, ll}

      bool inocrange = isInOCRange(Symbol(), giTf, i, MinBarsInRange, Range, aOCiR);
      
      h       = aOCiR[0];
      l       = aOCiR[1];
      ocrange = aOCiR[2];  
      hh      = aOCiR[3];
      ll      = aOCiR[4];    

      
      if (inocrange)//**
      {
            //Print("Found Range @ i = ", i);
            lasth =h; lastl=l; lastocr = ocrange; lasthh = hh; lastll = ll;
            for(j=1; j<=MaxBarsInRange-MinBarsInRange; j++)//***
            {
               inocrange = isInOCRange(Symbol(), giTf, i, MinBarsInRange+j, Range, aOCiR);

               if (!inocrange) break;
               else
               {
                  h       = aOCiR[0];
                  l       = aOCiR[1];
                  ocrange = aOCiR[2];  
                  hh      = aOCiR[3];
                  ll      = aOCiR[4];   
                    
                  lasth   = MathMax(h,       lasth); 
                  lastl   = MathMin(l,       lastl); 
                  lastocr = MathMax(ocrange, lastocr);         
                  lasthh  = MathMax(hh,      lasthh); 
                  lastll  = MathMin(ll,      lastll); 
                  

               }                              
            }//***
                        
            if (i+j != last_ij) last_ij = i+j;
            else continue;
            
            //put it back to h, l and ocrange
            h = lasth; l=lastl; ocrange = lastocr; hh = lasthh; ll = lastll;
            
            datetime t1 = iTime(NULL, giTf, i  + MinBarsInRange+j-1-1);            
            datetime t2 = iTime(NULL, giTf, i) + (ExtendLines*giTf*60);
            //weekend
            if (TimeDayOfWeek(t2)> 5 ) t2+= (2*1440*60); 
                        
            drawTl(PREFIX+"hr"+t1,  t1, h,  t2, h,  LineColor, STYLE_SOLID, LineWidth);               
            drawTl(PREFIX+"lr"+t1,  t1, l,  t2, l,  LineColor, STYLE_SOLID, LineWidth);   
             
            if (ShowHHLLlines) 
            {
               drawTl(PREFIX+"hh"+t1, t1, hh, t2, hh, LineColor, STYLE_DOT, LineWidth);  
               drawTl(PREFIX+"ll"+t1, t1, ll, t2, ll, LineColor, STYLE_DOT, LineWidth);
            }   

            drawBox (PREFIX+"box"+t1, t1, h, iTime(NULL, giTf, i-1) - (Period() * 60), l, BoxColor);       
                                            
            drawText(PREFIX+"r"+t1
               , DoubleToStr(ocrange,0) + "/" +Range + "/" + MinBarsInRange
               , t1
               , hh + 0.4*iATR(NULL, 0, 20, iBarShift(NULL, 0, iTime(NULL, giTf, i)))
               );
            
            //on startup draw all the box, 
            //and then .. only update the last box;
            if (!startup || BarsToScan == 0) break;
                                                   
      }//**                       
   }//*     
          
   startup = false; 
   
   //---- update every 60 seconds
   Sleep(60000);
   
   return(0);
   
  }
//+------------------------------------------------------------------+
//| isInOCRange function                                             |
//+------------------------------------------------------------------+
bool isInOCRange(string sym, int tf, int istart, int minbars, int maxrange, double &a[])
{
      
   double ho, hc, h, lo, lc, l, ocrange;
   double hh, ll;
   
   ArrayInitialize(a, 0);
   
   ho = iOpen (sym, tf, iHighest(sym, tf, MODE_OPEN,  minbars, istart));
   hc = iClose(sym, tf, iHighest(sym, tf, MODE_CLOSE, minbars, istart));
   hh = iHigh (sym, tf, iHighest(sym, tf, MODE_HIGH,  minbars, istart)); 
   h  = NormalizeDouble(MathMax(ho,hc), Digits);
   
   lo = iOpen (sym, tf, iLowest (sym, tf, MODE_OPEN,  minbars, istart));
   lc = iClose(sym, tf, iLowest (sym, tf, MODE_CLOSE, minbars, istart));
   ll = iLow  (sym, tf, iLowest (sym, tf, MODE_LOW,   minbars, istart));
   l  = NormalizeDouble(MathMin(lo,lc), Digits);      
   
   ocrange = NormalizeDouble( (h-l)/fix_point, 0 );   
   
   if (ocrange <= maxrange)
   {
      a[0] = h;
      a[1] = l;
      a[2] = ocrange;//MathCeil(ocrange);
      a[3] = hh;
      a[4] = ll;
      return(true);
   }
   
   return(false);
}
//+------------------------------------------------------------------+
//| drawTl function                                                  |
//+------------------------------------------------------------------+
void drawTl(string tlName, datetime t1, double v1, datetime t2, double v2, 
            color tlColor, int style = STYLE_SOLID, int width = 1, string desc="")
{
   if(ObjectFind(tlName) != 0)
   {
      ObjectCreate(
            tlName
            , OBJ_TREND
            , 0
            , t1
            , v1
            , t2
            , v2
            );      
   }else
   {
      ObjectMove(tlName, 0, t1, v1);   
      ObjectMove(tlName, 1, t2, v2);
   }
   ObjectSet(tlName, OBJPROP_COLOR, tlColor);
   ObjectSet(tlName, OBJPROP_RAY, false);
   ObjectSet(tlName, OBJPROP_STYLE, style);
   ObjectSet(tlName, OBJPROP_WIDTH, width);
   ObjectSetText(tlName, desc);
}  
//+------------------------------------------------------------------+
//| drawBox function                                                 |
//+------------------------------------------------------------------+
void drawBox(string name, datetime t1, double v1, datetime t2, double v2,  color clr)
{
   if(ObjectFind(name) != 0)
   {
      ObjectCreate(
            name
            , OBJ_RECTANGLE
            , 0
            , t1
            , v1
            , t2
            , v2
            );      
   }else
   {
      ObjectMove(name, 0, t1, v1);   
      ObjectMove(name, 1, t2, v2);
   }
   ObjectSet(name, OBJPROP_COLOR, clr);
   ObjectSet(name, OBJPROP_BACK, true);
   
}
//+------------------------------------------------------------------+
//| drawText function                                                |
//+------------------------------------------------------------------+
void drawText(string name, string text, datetime t1, double v1)
{
   if(ObjectFind(name) != 0)
   {
      ObjectCreate(
            name
            , OBJ_TEXT
            , 0
            , t1
            , v1
            );      
   }else
   {
      ObjectMove(name, 0, t1, v1);        
   }
   ObjectSetText(name,text,FontSize,"Arial Narrow", TextColor);
   
}
//+------------------------------------------------------------------+
//| getTFByName function                                             |
//+------------------------------------------------------------------+
int getTFByName(string TFName)
{
  int m;
  if      (TFName == "MN1") m = PERIOD_MN1;
  else if (TFName == "W1" ) m = PERIOD_W1;
  else if (TFName == "D1" ) m = PERIOD_D1;
  else if (TFName == "H4" ) m = PERIOD_H4;
  else if (TFName == "H1" ) m = PERIOD_H1;
  else if (TFName == "M30") m = PERIOD_M30;
  else if (TFName == "M15") m = PERIOD_M15;
  else if (TFName == "M5" ) m = PERIOD_M5;
  else if (TFName == "M1" ) m = PERIOD_M1;
  else m = Period();  
  
  return(m);  

}
//+------------------------------------------------------------------+